/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2009 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 *
 *******************************************************************************
 *
 * $Id$
 *
 ******************************************************************************/
package it.tidalwave.metadata;

import java.beans.PropertyChangeListener;
import java.lang.reflect.Method;
import javax.annotation.Nonnull;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
/* package */ final class MetadataUtils 
  {
    private MetadataUtils()
      {
      }

    /***************************************************************************
     *
     * 
     **************************************************************************/
    @Nonnull     
    public static <OptionType> void validateOptions (@Nonnull OptionType[] options) 
      {
        if (options == null)
          {
            throw new IllegalArgumentException("options is mandatory (pass a zero-sided array if you have no options)");
          }
        
        for (final OptionType option : options)
          {
            if (option == null)
              {
                throw new IllegalArgumentException("null option");
              }
          }
      }
    
    /***************************************************************************
     *
     * 
     **************************************************************************/
    @Nonnull
    // TODO: replace with Parameters.xxx
    public static <OptionType, Option extends OptionType> Option findOption (@Nonnull final OptionType[] options, 
                                                                             @Nonnull final Option default_, 
                                                                             @Nonnull final Class<Option> optionClass)
      {
        validateOptions(options);
          
        for (final OptionType option : options)
          {
            if (optionClass.isAssignableFrom(option.getClass()))
              {
                return (Option)option;   
              }
          }
        
        return default_;
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public static void addPropertyChangeListener (@Nonnull final Object bean, 
                                                  @Nonnull final PropertyChangeListener listener)
      {
        try
          {
            final Method method = bean.getClass().getMethod("addPropertyChangeListener", PropertyChangeListener.class);
            method.invoke(bean, listener);
          }
        catch (RuntimeException e)
          {
            throw e;
          }
        catch (Exception e)
          {
            throw new IllegalArgumentException(String.format("The given object of class %s does not support PropertyChangeListeners", bean.getClass()), e);
          }
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public static void removePropertyChangeListener (@Nonnull final Object bean, 
                                                     @Nonnull final PropertyChangeListener listener)
      {
        try
          {
            final Method method = bean.getClass().getMethod("removePropertyChangeListener", PropertyChangeListener.class);
            method.invoke(bean, listener);
          }
        catch (RuntimeException e)
          {
            throw e;
          }
        catch (Exception e)
          {
            throw new IllegalArgumentException(String.format("The given object of class %s does not support PropertyChangeListeners", bean.getClass()), e);
          }
      }
  }
